@@ -1,3 +1,4 @@
+import argparse
 import logging
 
 import borgmatic.logger
@@ -44,6 +45,7 @@ def resolve_archive_name(
         full_command,
         extra_environment=environment.make_environment(config),
         borg_local_path=local_path,
+        borg_exit_codes=config.get('borg_exit_codes'),
     )
     try:
         latest_archive = output.strip().splitlines()[-1]
@@ -137,15 +139,33 @@ def list_repository(
         local_path,
         remote_path,
     )
+    json_command = make_rlist_command(
+        repository_path,
+        config,
+        local_borg_version,
+        argparse.Namespace(**dict(rlist_arguments.__dict__, json=True)),
+        global_arguments,
+        local_path,
+        remote_path,
+    )
+    borg_exit_codes = config.get('borg_exit_codes')
+
+    json_listing = execute_command_and_capture_output(
+        json_command,
+        extra_environment=borg_environment,
+        borg_local_path=local_path,
+        borg_exit_codes=borg_exit_codes,
+    )
 
     if rlist_arguments.json:
-        return execute_command_and_capture_output(
-            main_command, extra_environment=borg_environment, borg_local_path=local_path
-        )
-    else:
-        execute_command(
-            main_command,
-            output_log_level=logging.ANSWER,
-            borg_local_path=local_path,
-            extra_environment=borg_environment,
-        )
+        return json_listing
+
+    flags.warn_for_aggressive_archive_flags(json_command, json_listing)
+
+    execute_command(
+        main_command,
+        output_log_level=logging.ANSWER,
+        extra_environment=borg_environment,
+        borg_local_path=local_path,
+        borg_exit_codes=borg_exit_codes,
+    )
