@@ -1,13 +1,14 @@
 import logging
 
-from borgmatic.execute import execute_command
+from borgmatic.borg import environment
+from borgmatic.execute import execute_command_and_capture_output
 
 logger = logging.getLogger(__name__)
 
 
-def local_borg_version(local_path='borg'):
+def local_borg_version(config, local_path='borg'):
     '''
-    Given a local Borg binary path, return a version string for it.
+    Given a configuration dict and a local Borg binary path, return a version string for it.
 
     Raise OSError or CalledProcessError if there is a problem running Borg.
     Raise ValueError if the version cannot be parsed.
@@ -17,7 +18,12 @@ def local_borg_version(local_path='borg'):
         + (('--info',) if logger.getEffectiveLevel() == logging.INFO else ())
         + (('--debug', '--show-rc') if logger.isEnabledFor(logging.DEBUG) else ())
     )
-    output = execute_command(full_command, output_log_level=None, borg_local_path=local_path)
+    output = execute_command_and_capture_output(
+        full_command,
+        extra_environment=environment.make_environment(config),
+        borg_local_path=local_path,
+        borg_exit_codes=config.get('borg_exit_codes'),
+    )
 
     try:
         return output.split(' ')[1].strip()
